"""
 Copyright 2024 - Minton Group at Purdue University
 This file is part of Swiftest.
 Swiftest is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 Swiftest is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
 of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 You should have received a copy of the GNU General Public License along with Swiftest. 
 If not, see: https://www.gnu.org/licenses. 
"""

# python functions to read in and set up spherical harmonics coefficients for non-standard gravity terms
# using pySHTOOLS see: https://shtools.github.io/SHTOOLS/
# 

from .constants import GC
import os

if 'READTHEDOCS' in os.environ:
    # Assume pyshtools is available when building on ReadTheDocs
    PYSHTOOLS_AVAILABLE = True
else:
    try:
        import pyshtools as pysh
        PYSHTOOLS_AVAILABLE = True
    except ModuleNotFoundError:
        PYSHTOOLS_AVAILABLE = False


if PYSHTOOLS_AVAILABLE:

    def clm_from_ellipsoid(mass, density, a, b = None, c = None, lmax = 6, lref_radius = False, ref_radius = None):
        """
        Creates and returns the gravity coefficients for an ellipsoid with principal axes a, b, c upto a certain maximum degree lmax. 
        Uses pyshtools. No units necessary for a, b, & c. However, they need to be in the same units (DU).

        Parameters
        ----------
        mass : float
            mass of the central body
        density : float
            density of the central body
        a : float 
            length of the pricipal axis aligned with the x axis
        b : float, optional, default = a
            length of the pricipal axis aligned with the y axis
        c : float, optional, default = b
            length of the pricipal axis aligned with the z axis
        lmax : int, optional, default = 6
            The maximum spherical harmonic degree resolvable by the grid.
        lref_radius : boolean, optional, default = False
            Boolean value to return the reference radius calculated by SHTOOLS
        ref_radius : float, optional, default = None
            Reference radius to scale the gravitational coefficients to

        Returns
        -------
        clm : ndarry, shape (2, lmax+1, lmax+1)
            numpy ndarray of the gravitational potential spherical harmonic coefficients. 
            This is a three-dimensional array formatted as coeffs[i, degree, order], 
            where i=0 corresponds to positive orders and i=1 to negative orders.

        """
        Gmass = GC * mass # SHTOOLS uses an SI G value, and divides it before using the mass; NO NEED TO CHANGE UNITS

        # cap lmax to ensure fast performance without giving up accuracy
        lmax_limit = 6              # lmax_limit = 6 derived from Jean's Law; characteristic wavelength = the radius of the CB
        if(lmax > lmax_limit):                           
            lmax = lmax_limit
            print(f'Setting maximum spherical harmonic degree to {lmax_limit}')

        # create shape grid 
        shape_SH = pysh.SHGrid.from_ellipsoid(lmax = lmax, a = a, b = b, c = c)

        # get gravity coefficients
        clm_class = pysh.SHGravCoeffs.from_shape(shape_SH, rho = density, gm = Gmass) # 4pi normalization
        clm = clm_class.to_array(normalization = '4pi') # export as array with 4pi normalization and not scaling by 4*pi to match normalisation

        # Return reference radius EQUALS the radius of the Central Body
        print(f'Ensure that the Central Body radius equals the reference radius.')

        if(lref_radius == True and ref_radius is None):
            ref_radius = shape_SH.expand(normalization = '4pi').coeffs[0, 0, 0]
            return clm, ref_radius
        elif(lref_radius == True and ref_radius is not None):
            clm_class = clm_class.change_ref(r0 = ref_radius)
            clm = clm_class.to_array(normalization = '4pi')
            return clm, ref_radius
        else:
            return clm

    def clm_from_relief(mass, density, grid, lmax = 6, lref_radius = False, ref_radius = None):
        """
        Creates and returns the gravity coefficients for a body with a given DH grid upto a certain maximum degree lmax. 
        Uses pyshtools.

        Parameters
        ----------
        mass : float
            mass of the central body
        density : float
            density of the central body
        grid : array, shape []
            DH grid of the surface relief of the body
        lmax : int, optional, default = 6
            The maximum spherical harmonic degree resolvable by the grid.
        lref_radius : boolean, optional, default = False
            Boolean value to return the reference radius calculated by SHTOOLS
        ref_radius : float, optional, default = None
            Reference radius to scale the gravitational coefficients to

        Returns
        -------
        clm : ndarry, shape (2, lmax+1, lmax+1)
            numpy ndarray of the gravitational potential spherical harmonic coefficients. 
            This is a three-dimensional array formatted as coeffs[i, degree, order], 
            where i=0 corresponds to positive orders and i=1 to negative orders.

        """

        Gmass = GC * mass # SHTOOLS uses an SI G value, and divides it before using the mass; NO NEED TO CHANGE UNITS

        # cap lmax to ensure fast performance
        lmax_limit = 6
        if(lmax > lmax_limit): # FIND A BETTER WAY to judge this cut off point, i.e., relative change between coefficients
            lmax = lmax_limit
            print(f'Setting maximum spherical harmonic degree to {lmax_limit}')

        # convert to spherical harmonics
        shape_SH = pysh.SHGrid.from_array(grid)

        # get coefficients
        clm_class = pysh.SHGravcoeffs.from_shape(shape_SH, rho = density, gm = Gmass) # 4pi normalization
        clm = clm_class.to_array(normalization = '4pi') # export as array with 4pi normalization

        # Return reference radius EQUALS the radius of the Central Body

        print(f'Ensure that the Central Body radius equals the reference radius.')

        if(lref_radius == True and ref_radius is None):
            ref_radius = shape_SH.expand(normalization = '4pi').coeffs[0, 0, 0]
            return clm, ref_radius
        elif(lref_radius == True and ref_radius is not None):
            clm_class = clm_class.change_ref(r0 = ref_radius)
            clm = clm_class.to_array(normalization = '4pi')
            return clm, ref_radius
        else:
            return clm

else:
    def clm_from_ellipsoid(*args, **kwargs):
        raise NotImplementedError("Sph_Harmonics is not available because pyshtools is not installed.")
    def clm_from_relief(*args, **kwargs):
        raise NotImplementedError("Sph_Harmonics is not available because pyshtools is not installed.")
    


import os
import pty
import subprocess
import sys

def main_caf():
    package_root = os.path.dirname(os.path.abspath(__file__))
    binary_path = os.path.join(package_root, "swiftest_caf")
    if os.path.exists(binary_path):
        main("swiftest_caf")
    else:
        print("This version of swiftest has not been compiled with coarray support. The standard version of swiftest will be executed instead.")
        main() 
    return

def main(binary_name="swiftest"):
    """Executes the 'swiftest' binary located in the package root, passing along any command-line arguments, and streams the output to the terminal in real-time, handling progress bars correctly by using a pseudo-terminal."""

    # Determine the path to the binary relative to this script
    package_root = os.path.dirname(os.path.abspath(__file__))
    binary_path = os.path.join(package_root, binary_name)

    # sys.argv[1:] contains all the arguments passed to the script, excluding the script name itself
    args = [binary_path] + sys.argv[1:]

    # Use pty to spawn the process and create a pseudo-terminal
    main_fd, subordinate_fd = pty.openpty()

    # Spawn the subprocess
    proc = subprocess.Popen(args, stdin=subordinate_fd, stdout=subordinate_fd, stderr=subordinate_fd, close_fds=True)

    # Close the subordinate file descriptor in the parent process
    os.close(subordinate_fd)

    # Read the output from the main file descriptor
    while True:
        try:
            output = os.read(main_fd, 1024).decode()
            if output:
                sys.stdout.write(output)
                sys.stdout.flush()
            else:
                break
        except OSError:
            break

    # Wait for the subprocess to finish
    proc.wait()

    return

if __name__ == "__main__":
    main()



#!/usr/bin/env python3
"""
 Copyright 2024 - The Minton Group at Purdue University
 This file is part of Swiftest.
 Swiftest is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 Swiftest is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
 of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 You should have received a copy of the GNU General Public License along with Swiftest. 
 If not, see: https://www.gnu.org/licenses. 
"""
import swiftest
import unittest
import os
import tempfile

class TestParticleType(unittest.TestCase):
    def setUp(self):
        # Initialize a target and surface for testing
        self.tmpdir=tempfile.TemporaryDirectory()
        self.simdir = self.tmpdir.name
        
    def tearDown(self):
        # Clean up temporary directory
        self.tmpdir.cleanup() 
        
    def test_correct_particle_types(self):
        '''
        Check that a variety of particles are set up with the valid particle types
        '''

        print('\ntest_correct_particle_types: Check that a variety of particles are set up with the valid particle types')

        valid_particle_types = [swiftest.constants.PL_TINY_TYPE_NAME, swiftest.constants.PL_TYPE_NAME, swiftest.constants.TP_TYPE_NAME, swiftest.constants.CB_TYPE_NAME]
        sim = swiftest.Simulation(simdir=self.simdir)

        sim.add_solar_system_body(['Sun'])

        # semi-interacting body
        sim.add_body(name=['pl_massive', 'pl_semi_int'], a=[1.5, 2.0], e=[0.1, 0.1], mass=[1e-11, 1e-6], radius=[1e-5, 1e-6])

        # test particle
        sim.add_body(name='tp', a=2.0, e=0.01)


        for particle_type in sim.init_cond.particle_type.values:
            self.assertTrue(particle_type in valid_particle_types, msg = f'Invalid particle type set up for {particle_type}')

        return 
         
if __name__ == '__main__':
    os.environ["HDF5_USE_FILE_LOCKING"]="FALSE"
    unittest.main()


"""
Copyright 2024 - The Minton Group at Purdue University
This file is part of Swiftest.
Swiftest is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
Swiftest is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
You should have received a copy of the GNU General Public License along with Swiftest. 
If not, see: https://www.gnu.org/licenses. 
"""

import astropy.constants as const
import datetime
import numpy as np

# Constants in SI units
GC = np.longdouble(const.G.value[()])
AU2M = np.longdouble(const.au.value)
GMSun = np.longdouble(const.GM_sun.value)
MSun = np.longdouble(const.M_sun.value)
RSun = np.longdouble(const.R_sun.value)
MEarth = np.longdouble(const.M_earth.value)
REarth = np.longdouble(const.R_earth.value)
GMEarth = np.longdouble(const.GM_earth.value)
JD2S = 86400
YR2S = np.longdouble(365.25 * JD2S)
einsteinC = np.longdouble(const.c) 
CB_TYPE_NAME = "Central Body"
PL_TYPE_NAME = "Massive Body"
TP_TYPE_NAME = "Test Particle"
PL_TINY_TYPE_NAME = "Semi-Interacting Massive Body"

# The default value is Prof. Minton's Brimley/Cocoon line crossing date (aka MBCL)
_mbday = datetime.date.fromisoformat('1976-08-05')
_bcl = datetime.timedelta(days=18530)
MINTON_BCL = (_mbday + _bcl).isoformat()



# cython: language_level=3, c_string_type=unicode, c_string_encoding=ascii
cimport cython
cimport numpy as cnp
import numpy as np
import warnings
import traceback

# Define this macro to use the NumPy C API version compatible with the build system's numpy version
cnp.import_array()

cdef extern from "core.h":
    void bindings_c_driver(char* integrator, char* param_file_name, char* display_style) nogil
    void bindings_orbel_el2xv(int nbody, double *mu, double *a, double *e, double *inc, double *capom, double *omega, double *capm, double *rx, double *ry, double *rz, double *vx, double *vy, double *vz) nogil
    void bindings_orbel_xv2el(int nbody, double *mu, double *rx, double *ry, double *rz, double *vx, double *vy, double *vz, double *a, double *e, double *inc, double *capom, double *omega, double *capm, double *varpi, double *lam, double *f, double *cape, double *capf) nogil

def driver(integrator, param_file_name, display_style):
    b_integrator = bytes(integrator,'ascii') + b'\x00'
    b_param_file_name = bytes(param_file_name,'ascii') + b'\x00'
    b_display_style = bytes(display_style,'ascii') + b'\x00'

    cdef:
        char* c_integrator = b_integrator 
        char* c_param_file_name = b_param_file_name 
        char* c_display_style = b_display_style 

    try:
        with nogil:
            bindings_c_driver(c_integrator, c_param_file_name, c_display_style)
    except Exception as e:  
        traceback_details = traceback.format_exc()
        warning_message = f"An unexpected error occurred in bindings_c_driver: {e}\n{traceback_details}"
        warnings.warn(warning_message, RuntimeWarning)

    return

def el2xv(cnp.ndarray[cnp.float64_t, ndim=1] mu,
          cnp.ndarray[cnp.float64_t, ndim=1] a,
          cnp.ndarray[cnp.float64_t, ndim=1] e,
          cnp.ndarray[cnp.float64_t, ndim=1] inc,
          cnp.ndarray[cnp.float64_t, ndim=1] capom,
          cnp.ndarray[cnp.float64_t, ndim=1] omega,
          cnp.ndarray[cnp.float64_t, ndim=1] capm):
    """
    Convert orbital elements to state vectors

    Parameters
    ----------
    mu : (N,) array of floats
        Gravitational parameter
    a : (N,) array of floats
        Semi-major axis
    e : (N,) array of floats
        Eccentricity
    inc : (N,) array of floats
        Inclination (degrees)
    capom : (N,) array of floats
        Longitude of the ascending node (degrees)
    omega : (N,) array of floats
        Argument of periapsis (degrees)
    capm : (N,) array of floats
        Mean anomaly (degrees)

    Returns
    -------
    rh : (N,3) array of floats
        position vector
    vh : (N,3) array of floats
    """ 
    if not (mu.size == a.size == e.size == inc.size == capom.size == omega.size == capm.size):
        raise ValueError("All input arrays must have the same length")

    cdef int nbody = mu.size

    # Ensure memory-contiguous numpy arrays
    mu = np.ascontiguousarray(mu, dtype=np.float64)
    mu.flags.writeable = True  
    a = np.ascontiguousarray(a, dtype=np.float64)
    a.flags.writeable = True
    e = np.ascontiguousarray(e, dtype=np.float64)
    e.flags.writeable = True
    inc_rad = np.ascontiguousarray(inc, dtype=np.float64)
    inc_rad.flags.writeable = True
    capom_rad = np.ascontiguousarray(capom, dtype=np.float64)
    capom_rad.flags.writeable = True
    omega_rad = np.ascontiguousarray(omega, dtype=np.float64)
    omega_rad.flags.writeable = True
    capm_rad = np.ascontiguousarray(capm, dtype=np.float64)
    capm_rad.flags.writeable = True

    # Make memory view of the numpy arrays and convert angular quantities to radians
    cdef cnp.float64_t[::1] mu_v = mu
    cdef cnp.float64_t[::1] a_v = a
    cdef cnp.float64_t[::1] e_v = e
    cdef cnp.float64_t[::1] inc_v = inc_rad
    cdef cnp.float64_t[::1] capom_v = capom_rad
    cdef cnp.float64_t[::1] omega_v = omega_rad
    cdef cnp.float64_t[::1] capm_v = capm_rad

    # Create arrays for outputs
    _rx = np.empty(nbody, dtype=np.float64)
    _ry = np.empty(nbody, dtype=np.float64)
    _rz = np.empty(nbody, dtype=np.float64)
    _vx = np.empty(nbody, dtype=np.float64)
    _vy = np.empty(nbody, dtype=np.float64)
    _vz = np.empty(nbody, dtype=np.float64)

    cdef cnp.float64_t[::1] rx = _rx 
    cdef cnp.float64_t[::1] ry = _ry
    cdef cnp.float64_t[::1] rz = _rz
    cdef cnp.float64_t[::1] vx = _vx
    cdef cnp.float64_t[::1] vy = _vy
    cdef cnp.float64_t[::1] vz = _vz

    with cython.boundscheck(False):
        with nogil:
            bindings_orbel_el2xv(nbody, &mu_v[0], &a_v[0], &e_v[0], &inc_v[0], &capom_v[0], &omega_v[0], &capm_v[0], &rx[0], &ry[0], &rz[0], &vx[0], &vy[0], &vz[0])

    cdef cnp.ndarray[cnp.float64_t, ndim=2] rh = np.stack((rx, ry, rz), axis=1)
    cdef cnp.ndarray[cnp.float64_t, ndim=2] vh = np.stack((vx, vy, vz), axis=1)

    return rh, vh


def xv2el(cnp.ndarray[cnp.float64_t, ndim=1] mu,
          cnp.ndarray[cnp.float64_t, ndim=2] rh,
          cnp.ndarray[cnp.float64_t, ndim=2] vh):
    """
    Convert state vectors to orbital elements

    Parameters
    ----------
    mu : (N,) array of floats
        Gravitational parameter
    rh : (N,3) array of floats
        position vector
    vh : (N,3) array of floats

    Returns
    -------
    a : (N,) array of floats
        Semi-major axis
    e : (N,) array of floats
        Eccentricity
    inc : (N,) array of floats
        Inclination (degrees)
    capom : (N,) array of floats
        Longitude of the ascending node (degrees)
    omega : (N,) array of floats
        Argument of periapsis (degrees)
    capm : (N,) array of floats
        Mean anomaly (degrees)
    varpi : (N,) array of floats
        Longitude of periapsis (degrees)
    lam : (N,) array of floats
        True longitude (degrees)
    f : (N,) array of floats
        True anomaly (degrees)
    cape : (N,) array of floats
        Eccentric anomaly (degrees)
    capf : (N,) array of floats
        Eccentric true anomaly (degrees)
    """ 

    # Ensure the input arrays are compatible
    if not (mu.size == rh.shape[0] == vh.shape[0]):
        raise ValueError("mu, rh, and vh must have compatible dimensions.")

    cdef int nbody = mu.size

    # Ensure memory-contiguous numpy arrays
    mu = np.ascontiguousarray(mu, dtype=np.float64)
    mu.flags.writeable = True  
    rh.flags.writeable = True  
    vh.flags.writeable = True  

    # Make memory view of the numpy arrays
    cdef cnp.float64_t[::1] mu_v = mu
    cdef cnp.float64_t[::1] rx_v = np.ascontiguousarray(rh[:, 0], dtype=np.float64)
    cdef cnp.float64_t[::1] ry_v = np.ascontiguousarray(rh[:, 1], dtype=np.float64)
    cdef cnp.float64_t[::1] rz_v = np.ascontiguousarray(rh[:, 2], dtype=np.float64)
    cdef cnp.float64_t[::1] vx_v = np.ascontiguousarray(vh[:, 0], dtype=np.float64)
    cdef cnp.float64_t[::1] vy_v = np.ascontiguousarray(vh[:, 1], dtype=np.float64)
    cdef cnp.float64_t[::1] vz_v = np.ascontiguousarray(vh[:, 2], dtype=np.float64)
 
    # Create arrays for outputs
    _a = np.empty(nbody, dtype=np.float64)
    _e = np.empty(nbody, dtype=np.float64)
    _inc = np.empty(nbody, dtype=np.float64)
    _capom = np.empty(nbody, dtype=np.float64)
    _omega = np.empty(nbody, dtype=np.float64)
    _capm = np.empty(nbody, dtype=np.float64)
    _varpi = np.empty(nbody, dtype=np.float64)
    _lam = np.empty(nbody, dtype=np.float64)
    _f = np.empty(nbody, dtype=np.float64)
    _cape = np.empty(nbody, dtype=np.float64)
    _capf = np.empty(nbody, dtype=np.float64)

    cdef cnp.float64_t[::1] a = _a
    cdef cnp.float64_t[::1] e = _e
    cdef cnp.float64_t[::1] inc = _inc
    cdef cnp.float64_t[::1] capom = _capom
    cdef cnp.float64_t[::1] omega = _omega
    cdef cnp.float64_t[::1] capm = _capm
    cdef cnp.float64_t[::1] varpi = _varpi
    cdef cnp.float64_t[::1] lam = _lam
    cdef cnp.float64_t[::1] f = _f
    cdef cnp.float64_t[::1] cape = _cape
    cdef cnp.float64_t[::1] capf = _capf

    with cython.boundscheck(False):
        with nogil:
            bindings_orbel_xv2el(nbody, &mu_v[0], 
                                &rx_v[0], &ry_v[0], &rz_v[0], 
                                &vx_v[0], &vy_v[0], &vz_v[0], 
                                &a[0], &e[0], &inc[0], &capom[0], &omega[0], &capm[0], &varpi[0], &lam[0], &f[0], &cape[0], &capf[0])

    # Convert angular quantities to degrees 
    cdef cnp.ndarray[cnp.float64_t, ndim=1] a_np = np.asarray(a)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] e_np = np.asarray(e)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] inc_np = np.asarray(inc)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] capom_np = np.asarray(capom)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] omega_np = np.asarray(omega)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] capm_np = np.asarray(capm)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] varpi_np = np.asarray(varpi)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] lam_np = np.asarray(lam)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] f_np = np.asarray(f)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] cape_np = np.asarray(cape)
    cdef cnp.ndarray[cnp.float64_t, ndim=1] capf_np = np.asarray(capf)

    for i in range(nbody):
        if (rx_v[i] == 0.0 and ry_v[i] == 0.0 and rz_v[i] == 0.0 and vx_v[i] == 0.0 and vy_v[i] == 0.0 and vz_v[i] == 0.0):
            a_np[i] = np.nan
            e_np[i] = np.nan
            inc_np[i] = np.nan
            capom_np[i] = np.nan
            omega_np[i] = np.nan
            capm_np[i] = np.nan
            varpi_np[i] = np.nan
            lam_np[i] = np.nan
            f_np[i] = np.nan
            cape_np[i] = np.nan
            capf_np[i] = np.nan


    return a_np, e_np, inc_np, capom_np, omega_np, capm_np, varpi_np, lam_np, f_np, cape_np, capf_np





#!/usr/bin/env python3
"""
 Copyright 2024 - The Minton Group at Purdue University
 This file is part of Swiftest.
 Swiftest is free software: you can redistribute it and/or modify it under the terms of the GNU General Public License 
 as published by the Free Software Foundation, either version 3 of the License, or (at your option) any later version.
 Swiftest is distributed in the hope that it will be useful, but WITHOUT ANY WARRANTY; without even the implied warranty 
 of MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General Public License for more details.
 You should have received a copy of the GNU General Public License along with Swiftest. 
 If not, see: https://www.gnu.org/licenses. 
"""
import swiftest
import unittest
import os
import tempfile
import numpy as np

class TestXV2EL2XV(unittest.TestCase):
    def setUp(self):
        # Initialize a target and surface for testing
        self.tmpdir=tempfile.TemporaryDirectory()
        self.simdir = self.tmpdir.name
        
    def tearDown(self):
        # Clean up temporary directory
        self.tmpdir.cleanup() 
        
    def test_xv2el2xv_dims(self):
        '''
        Check that the xv2el and el2xv converts correctly with the right dimensions
        '''

        print('\ntest_xv2el2xv_dims: Check that the xv2el and el2xv converts with the right dimensions')

        sim = swiftest.Simulation(simdir=self.simdir, tstop = 0.5, dt = 0.1)

        sim.add_solar_system_body(['Sun', 'Mars'])
        xv_dims = sim.data.rh.dims
        el_dims = sim.data.a.dims

        # make dummy changes to the bodies
        sim.modify_body(name = 'Sun', c_lm = np.ones([2, 3, 3]))
        sim.modify_body(name = 'Mars', e = 0.1)
        sim.data.el2xv()
        sim.data.xv2el()

        # check that rh and vh have the same dimensions
        self.assertTrue(sim.data.rh.dims == xv_dims)
        self.assertTrue(sim.data.vh.dims == xv_dims)

        # check that all orbital elements have the same dimensions
        self.assertTrue(sim.data.a.dims == el_dims)
        self.assertTrue(sim.data.e.dims == el_dims)
        self.assertTrue(sim.data.inc.dims == el_dims)
        self.assertTrue(sim.data.omega.dims == el_dims)
        self.assertTrue(sim.data.capom.dims == el_dims)
        self.assertTrue(sim.data.capm.dims == el_dims)
        self.assertTrue(sim.data.capf.dims == el_dims)
        self.assertTrue(sim.data.cape.dims == el_dims)
        self.assertTrue(sim.data.varpi.dims == el_dims)
        self.assertTrue(sim.data.lam.dims == el_dims)

        return 
       
    def test_type_mismatch(self):
        ''' 
        Checks that dtypes are converted properly before passing arguments to the el2xv or xv2el methods 
        '''  
        sim = swiftest.Simulation(simdir=self.simdir)
        sim.add_solar_system_body(["Sun","Mercury"])
        GMcb = swiftest.GMSun * sim.M2DU**3 / sim.S2TU**2
        sim.data.xv2el(GMcb)
        sim.data.el2xv(GMcb)
        
if __name__ == '__main__':
    os.environ["HDF5_USE_FILE_LOCKING"]="FALSE"
    unittest.main()